#ifndef ANIMHANDLER_H
#define ANIMHANDLER_H

#include <QThread>
#include <QDebug>
#include <QFile>
#include <cstring>
#include "animation.h"
#include "spisender.h"

enum Mode {STOP=0, RUN, PAUSE};
class AnimHandler: public QThread
{
    Q_OBJECT
    void run() override;
private:
    uint32_t progress = 0;
    std::atomic<bool> exitThread {false};
    enum Mode mode = STOP;
    std::atomic<enum Mode> nextmode {STOP};
    std::atomic<bool> changemode {false};
    Animation anim;
    SpiSender spisender;

public:
    AnimHandler();
    void openFile(const std::string filepath);
    uint32_t getAnimTimeLen() const;

public slots:
    void start_slot();
    void pause();
    void stop();
    void enableExit();

signals:
    void onPaused();
    void onStarted();
    void onStopped();
    void onFileOpened(const std::string filepath);
    void updateFrame(MxFrame& frame);
    void progressUpdate(uint32_t progress);
    void resultReady(const QString &result);
};

class Alma: public QObject
{
    Q_OBJECT
AnimHandler* animhandler;
public:
    Alma() {
        animhandler = new AnimHandler;
        connect(this, &Alma::exit, animhandler, &AnimHandler::enableExit);
        animhandler->start();
        emit exit();
    }

    ~Alma() {
    }

signals:
    void exit();
};



#endif // ANIMHANDLER_H
