#include <filebrowser.h>
#include <sstream>
#include <QDir>
#include <QDebug>
#include <QDialog>
#include <QScrollBar>

using namespace std;
FileBrowser::FileBrowser( QWidget *parent)
        : QWidget(parent)
{

    scrollArea = new QScrollArea( this );
    scrollArea->setVerticalScrollBarPolicy( Qt::ScrollBarAlwaysOn );
    scrollArea->setWidgetResizable( true );
    scrollArea->setGeometry( 10, 10, 400, 400 );

    QWidget *widget = new QWidget();
    scrollArea->setWidget( widget );
    widget->setMaximumWidth(scrollArea->width()-20);

    layout = new QVBoxLayout();
    widget->setLayout( layout );
    layout->setAlignment(Qt::AlignTop);

    for (int i = 0; i < 10; i++)
    {
        QPushButton *button = new QPushButton( QString( "%1" ).arg( i ) );
        layout->addWidget( button );
    }

    setPath("/home/tomi/Documents/minimatrixrpi");
}

FileBrowser::~FileBrowser()
{
}

void FileBrowser::update()
{
//    return;
    QLayoutItem* item;
    while ( ( item = layout->takeAt( 0 ) ) != NULL )
    {
        delete item->widget();
        delete item;
    }

    buttons.clear();
    if (canCd(getCdUp()))
    {
        buttons.push_back(new QPushButton(".."));
        layout->addWidget(buttons.back());
        connect(buttons.back(), SIGNAL(clicked()), this, SLOT(backClicked()));
    }
    vector<string> dirs = getDirs(path);
    for (int i = 0; i < dirs.size(); i++) {
        buttons.push_back(new QPushButton(dirs[i].c_str()));
        layout->addWidget(buttons.back());
        connect(buttons.back(), SIGNAL(clicked()), this, SLOT(dirClicked()));
    }
    vector<string> files= getFiles(path);
    for (int i = 0; i < files.size(); i++) {
        buttons.push_back(new QPushButton(files[i].c_str()));
        layout->addWidget(buttons.back());
        connect(buttons.back(), SIGNAL(clicked()), this, SLOT(fileClicked()));
        QPalette pal = buttons.back()->palette();
        pal.setColor(QPalette::Button, QColor(Qt::cyan));
        buttons.back()->setAutoFillBackground(true);
        buttons.back()->setPalette(pal);
        buttons.back()->update();
    }

    for (int i = 0; i < buttons.size(); i++)
    {
        buttons[i]->setStyleSheet("Text-align:left");
    }
    scrollArea->verticalScrollBar()->setValue(0);
}

vector<string> FileBrowser::getDirs(string path)
{
    vector<string> files;
    QDir directory(path.c_str());
    QStringList images = directory.entryList(QStringList() << "*",QDir::Dirs | QDir::NoDotAndDotDot);
    foreach(QString filename, images) {
        files.push_back(filename.toStdString());
    }
    return files;
}

vector<string> FileBrowser::getFiles(string path)
{
    vector<string> files;
    QDir directory(path.c_str());
    QStringList images = directory.entryList(QStringList() << "*.qp4", QDir::Files);
    foreach(QString filename, images) {
        files.push_back(filename.toStdString());
    }
    return files;
}

void FileBrowser::dirClicked()
{
    QPushButton* bt = (QPushButton*)sender();
    string newpath = path + "/" + bt->text().toStdString();
    qDebug() << newpath.c_str();
    setPath(newpath);
}

void FileBrowser::backClicked()
{
    string newPath = getCdUp();
    qDebug() << newPath.c_str();
    if (canCd(newPath))
        setPath(newPath);
}

void FileBrowser::fileClicked()
{
    QPushButton* bt = (QPushButton*)sender();
    qDebug() << "File: " << bt->text().toStdString().c_str();
    const string filepath = path + "/" + bt->text().toStdString();
    fileSelected(filepath);
}

void FileBrowser::setPath(std::string newpath)
{
    path = newpath;
    update();
}

const string FileBrowser::getPath()
{
    return this->path;
}

string FileBrowser::getCdUp() const
{
    QDir dir(path.c_str());
    dir.cdUp();
    return dir.cleanPath(dir.path()).toStdString();
}

bool FileBrowser::canCd(string path) const
{
    if (allowedFolders.size() == 0) return true;
    QDir dir(path.c_str());
    string apath = dir.absolutePath().toStdString();
    for (size_t i = 0; i < allowedFolders.size(); i++) {
        if (apath.rfind(allowedFolders[i], 0) == 0)
        {
            return true;
        }
    }
    return false;
}







