#include "mainwindow.h"
#include "ui_mainwindow.h"
#include <stdint.h>

MainWindow::MainWindow(QWidget *parent)
    : QDialog(parent)
    , ui(new Ui::MainWindow)
{
    ui->setupUi(this);

    btRun = new QPushButton("Run", this);
    btPause = new QPushButton("Pause", this);
    btStop = new QPushButton("Stop", this);
    ui->gridLayout->addWidget(btRun, 0, 0);
    ui->gridLayout->addWidget(btPause, 0, 1);
    ui->gridLayout->addWidget(btStop, 0, 2);

    fb = new FileBrowser();
    animhandler = new AnimHandler();
    animhandler->start();
    //animhandler->openFile("/home/tomi/Documents/minimatrixrpi/asdasd.qp4");
    mxemu = new MxEmu();

    progressbar = new QProgressBar(this);
    progressbar->setRange(0,100);
    progressbar->setValue(10);
    progressbar->setTextVisible(false);

    fb->allowedFolders.push_back("/home/tomi/Documents/minimatrixrpi");
    fb->update();
    ui->gridLayout->addWidget(fb, 1, 0, 1, 2);
    ui->gridLayout->addWidget(mxemu, 1, 2);
    ui->gridLayout->addWidget(progressbar, 2, 0, 1, 3);

    qRegisterMetaType<uint32_t>("uint32_t");
    qRegisterMetaType<MxFrame>("MxFrame&");
    connect(btRun, SIGNAL(clicked()), animhandler, SLOT(start_slot()));
    connect(btPause, SIGNAL(clicked()), animhandler, SLOT(pause()));
    connect(btStop, SIGNAL(clicked()), animhandler, SLOT(stop()));
    connect(animhandler, &AnimHandler::updateFrame, mxemu, &MxEmu::updateFrame);
    connect(fb, &FileBrowser::fileSelected, animhandler, &AnimHandler::openFile);
    connect(animhandler, &AnimHandler::progressUpdate, this, &MainWindow::progressupdate);
    connect(animhandler, &AnimHandler::onFileOpened, this, &MainWindow::fileopened);
}

void MainWindow::fileopened(const std::string filepath)
{
    qDebug() << "File opened: " << filepath.c_str();
    progressbar->setValue(0);
    progressbar->setMaximum(animhandler->getAnimTimeLen());
    progressbar->update();
}

void MainWindow::progressupdate(uint32_t t)
{
    progressbar->setValue(t);
    progressbar->update();
}

MainWindow::~MainWindow()
{
    delete ui;
}

