#include <QtGui>
#include "mxemu.h"

MxEmu::MxEmu(QWidget *parent) : QWidget(parent)
{
    for (int y = 0; y < 26; y++) {
        for (int x = 0; x < 32; x++) {
            frame.pixels[y][x] = Color(255,255,255);
        }
    }
}

QSize MxEmu::minimumSizeHint() const
{
    return QSize(100, 100);
}

QSize MxEmu::sizeHint() const
{
    return QSize(100, 200);
}

void MxEmu::paintEvent(QPaintEvent * /* event */)
{
    const int pixSize = 5;
    const int vWindowMargin = 8;
    const int hWindowMargin = 2;
    const int vPixNum = 26;
    const int hPixNum = 32;
    const int windowBorderWidth = 1;
    const int fullWidth = hPixNum * pixSize + 3 * hPixNum / 2 * windowBorderWidth + (hPixNum/2 + 1) * hWindowMargin;
    const int fullHeight = vPixNum * pixSize + 3 * vPixNum / 2 * windowBorderWidth + (vPixNum/2 + 1) * vWindowMargin;

    QPainter painter(this);
    painter.setPen(QColor(0,0,0));
    painter.setBrush(QColor(80,80,80));
    painter.drawRect(0,0,fullWidth+1,fullHeight+1);

    QPen pen(QColor(0,0,0));
    pen.setWidth(windowBorderWidth);
    if (windowBorderWidth <= 0) pen.setStyle(Qt::PenStyle::NoPen);
    painter.setPen(pen);
    for (int y = 0; y < 26; y++) {
        for (int x = 0; x < 32; x++) {
            Color c = frame.pixels[y][x];
            painter.setBrush(QColor(c.r, c.g, c.b, c.a));
            int x0 = 1+hWindowMargin * (x/2+1) + x*pixSize + (x + x/2) * windowBorderWidth;
            int y0 = 1+vWindowMargin * (y/2+1) + y*pixSize + (y + y/2) * windowBorderWidth;

            painter.drawRect(x0, y0, pixSize+windowBorderWidth, pixSize+windowBorderWidth);
        }
    }
}

void MxEmu::updateFrame(MxFrame& frame)
{
    this->frame = frame;
    update();
}
