// Server side implementation of UDP client-server model
#include "matrix_server.h"
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <stdint.h>
#include <fcntl.h>

MatrixServer::MatrixServer()
{
    buffer = (uint8_t*)malloc(BUFFER_SIZE);
    if ((sockfd = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
    {
        perror("socket creation failed");
        exit(EXIT_FAILURE);
    }
    int flags = fcntl(sockfd, F_GETFL);
    flags |= O_NONBLOCK;
    fcntl(sockfd, F_SETFL, flags);

    memset(&servaddr, 0, sizeof(servaddr));
    memset(&cliaddr, 0, sizeof(cliaddr));

    // Filling server information
    servaddr.sin_family = AF_INET; // IPv4
    servaddr.sin_addr.s_addr = INADDR_ANY;
    servaddr.sin_port = htons(PORT);

    // Bind the socket with the server address
    if (bind(sockfd, (const struct sockaddr *)&servaddr,
             sizeof(servaddr)) < 0)
    {
        perror("bind failed");
        exit(EXIT_FAILURE);
    }
}

MatrixServer::~MatrixServer()
{
    free(buffer);
}

bool MatrixServer::receive()
{
    size_t len = sizeof(cliaddr); //len is value/result
    datagram_size = recvfrom(sockfd, (char *)buffer, BUFFER_SIZE,
                 MSG_WAITALL, (struct sockaddr *)&cliaddr,
                 &len);
    if (datagram_size < 0)
    {
        // No new data.
        return false;
    }
    return parseDatagram(buffer, datagram_size);
}

bool MatrixServer::parseDatagram(uint8_t* buffer, size_t len)
{
    if (len != 1250)
        return false;
    // Protocol type must be 2.
    if (buffer[0] != 2)
        return false;
    frame = MxFrame();
    int k = 0;
    int x = 0;
    int y = 0;
    for (size_t i = 2; i < len; i++)
    {
        uint8_t color = buffer[i];
        uint8_t colorH = (color >> 4) & 0x07;
        uint8_t colorL = color & 0x07;
        if (k % 3 == 0)
        {
            frame.pixels[y][x].r = colorH;
            frame.pixels[y][x].g = colorL;
        }
        else if (k % 3 == 1)
        {
            frame.pixels[y][x].g = colorH;
            frame.pixels[y][x].b = colorL;
            // Next 4 bits corresponds to the next pixel
            x++;
            if (x >= 32)
            {
                x = 0;
                y++;
            }
        }
        else if (k % 3 == 2)
        {
            frame.pixels[y][x].b = colorH;
            // Next 4 bits corresponds to the next pixel
            x++;
            if (x >= 32)
            {
                x = 0;
                y++;
            }
            frame.pixels[y][x].r = colorL;
        }
        k = (k + 2) % 3;
    }
    return true;
}

MxFrame MatrixServer::getFrame()
{
    return frame;
}
